/*************************************************************
 * (C) 2001, Roberto A. Foglietta <robang@libero.it>
 *           Tutti i diritti riservati oppure GPL.
 *  
 * (C) 2001, Roberto A. Foglietta <robang@libero.it>
 *           Delivered under GPL as beta software,
 *           use at your own risk.
 *           
 * http://digilander.iol.it/robang
 *                    
 * rafobj.c : nuovo modulo gestione oggetti  
 * creazione: 2001-04-25 
 * modifiche: 2001-05-28 v0.1 
 * 						2001-05-29 v0.1.1 BUG FIX & TESTED
 * 						2001-05-30 v0.1.2 some GET call SLT
 * 						2001-06-22 v0.1.3 NEXT/PREV return NULL at the end of buf
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "rafobj.h"
										
const void *raf_obj_manage(int flag, void *pAram)
{
	RAF_DEBUG_INIT{"raf_obj_manage"};
	/* RAF 2001-05-23: questa funzione si comporta come 
	 * un C++ Object dove i flags fungono da metodi.
	 */
	static int prima = 1;
	static unsigned NumObj = 0;
	static unsigned Freeze = 0;
	static void *ListObj[RAF_MAX_NUM_OBJECT];
	static long  ListCod[RAF_MAX_NUM_OBJECT];
	static char  ListDel[RAF_MAX_NUM_OBJECT];
	void *v;
	long u;
	int i;

	/* RAF 2001-05-25: inizializzazione */
	if(prima) {
		for(i = 0; i < RAF_MAX_NUM_OBJECT; i++) {
			ListObj[i] = NULL;
			ListCod[i] = 0;
			ListDel[i] = 0;
		}
	 	prima = 0;
	}
	
	switch(flag & 0xF0) {
		case RAF_OBJ_ADD:
			/* RAF 2001-05-25: 
			 * genera un codice identificativo univoco 
			 */
			u = raf_object_get_unq;
			RAF_DEBUG("RAF_OBJ_ADD aggiunto %ld", u);
			ListCod[NumObj] = u;
			
			/* RAF 2001-05-28: 
			 * genera un copia dell'oggetto passatogli
			 * TODO: il cast non viene eseguito ma solo 
			 * inizializzato anche se per ora e` solo 
			 * una costante 
			 */
			ListObj[NumObj] = (void *)raf_obj_manage(RAF_OBJ_CPY, pAram);
			NumObj++;

			/* RAF 2001-05-25: 
			 * ritorna il codice identificativo univoco 
			 */
			return (void *)u;
			break;
			
		case RAF_OBJ_CPY:
			/* RAF 2001-05-28:
			 * genera una nuova istanza dell'oggetto 
			 */
			v = (void *)malloc(sizeof(RAF_TYPE_OBJECT));
			v = memcpy(v, pAram, sizeof(RAF_TYPE_OBJECT));		
			return v;
			break;
			
		case RAF_OBJ_RMV:
			/* RAF 2001-05-28:
			 * In alcune situazioni il contenuto deve essere 
			 * accessibile per un certo periodo anche se nel 
			 * contempo alcuni dovrebbero essere eliminati.
			 * Con il freeze/thaw i delete sono posticipati
			 */
			switch(flag & 0x0F) {
				case RAF_FREEZE:
					Freeze++;
					if(!Freeze) 
					 Freeze = (unsigned)-1;
					return (void *)Freeze;
					break;

				case RAF_THAW:
					if(Freeze > 0)
					 Freeze--;
					/* RAF 2001-05-28:
					 * Elimino quelli che sono stati  
					 * accodati durante il freezing time
					 */
					if(!Freeze) {
						for(i = 0; i < NumObj; i++) 
							if(ListDel[i])
								raf_obj_manage(RAF_OBJ_RMV | RAF_IND, (void *)i);
					}
					return (void *)Freeze;
					break;
			}
			
			switch(flag & 0x0F) {
				case RAF_COD:
					/* RAF 2001-05-28: 
				 	 * Lo rimuovo per codice identificativo
			 		 */
					u = (long)pAram;
					if( raf_obj_manage(RAF_OBJ_SLT | RAF_COD, pAram) ) 
						raf_obj_manage(RAF_OBJ_RMV, NULL);
					else { 
						RAF_DEBUG("RAF_OBJ_RMV non trovato %ld", u); 
					}
					break;

				case RAF_IND:
					/* RAF 2001-05-28: 
				 	 * Lo rimuovo per indice (dall'esterno)
			 		 */
					i = (unsigned)pAram;
					if( raf_obj_manage(RAF_OBJ_SLT | RAF_COD, (void *)ListCod[i]) ) 
						raf_obj_manage(RAF_OBJ_RMV, NULL);
					else { 
						RAF_DEBUG("RAF_OBJ_RMV non trovato %ld", u); 
					}
					break;
				
				default:
					i = raf_object_get_ind;
					if(!Freeze)	{
						/* RAF 2001-05-28:
					 	 * Elimino _veramente_ quello selezionato
					 	 */
						RAF_DEBUG("RAF_OBJ_RMV rimozione %d cod. %ld", i, ListCod[i]);
						free(ListObj[i]);
						for(; i < NumObj-1; i++) {
							ListObj[i] = ListObj[i+1];
							ListCod[i] = ListCod[i+1];
							ListDel[i] = ListDel[i+1];
						}
						ListObj[i] = NULL;
						ListCod[i] = 0;
						ListDel[i] = 0;
						NumObj--;
					}	else {
						/* RAF 2001-05-28: 
				 		 * Invece di eliminare, accodo l'elemento
			 	 		 */
						RAF_DEBUG("RAF_OBJ_RMV accodato per l'eliminazione");
						ListDel[i] = 1;
					}
					break;
			} 
			return (void *)NumObj;	
			break;
	}

	switch(flag & 0xF0) {
		/* RAF 2001-05-25: per maggiore astrazione l'indice
		 * e` disponibile solo per i metodi SP_SLT e SP_GET 
		 */
 		static unsigned Select = 0;
		
		case RAF_OBJ_SLT:
			RAF_DEBUG("RAF_OBJ_SLT");
			switch(flag & 0x0F) {
				case RAF_FIRST:
					Select = 0;
					break;
				
				case RAF_PREV:
					if(Select > 0) Select--;
					/* RAF 2001-06-22 serve per il while!NULL PREV*/
					else return NULL;		
					break;

				case RAF_NEXT:
					if(Select+1 < NumObj) Select++;
					/* RAF 2001-06-22 serve per il while!NULL NEXT*/
					else return NULL;		
					break;
					
				case RAF_LAST:
					Select = NumObj-1;
					break;
				
				case RAF_COD:
					/* RAF 2001-05-28: (long)pAram
					 * Seleziona un elemento ricercandolo per codice 
					 */ 
					u = (long)pAram;
					for(i = 0; i < NumObj; i++)
						if(ListCod[i] == u) 
							break;
					if(i < NumObj) {
							RAF_DEBUG("RAF_OBJ_SLT trovato %lu in pos. %d/%d (0x%lx)", 
												u, i, NumObj, (long)ListObj[i]);
							Select = i;
					} else { 
							RAF_DEBUG("RAF_OBJ_SLT non trovato %ld", u); 
							return NULL;
					}
					break;
			}
			v = ListObj[Select];
			break;
		
		case RAF_OBJ_GET:
			switch(flag & 0x0F) {
				case RAF_IND:
					RAF_DEBUG("RAF_OBJ_GET | RAF_IND");
					v = (void *)Select;
					break;
					
				case RAF_NUM:
					RAF_DEBUG("RAF_OBJ_GET | RAF_NUM");
					v = (void *)NumObj;
					break;
					
				case RAF_COD:
					RAF_DEBUG("RAF_OBJ_GET | RAF_COD");
					v = (void *)(ListCod[Select]);
					break;

				case RAF_FREEZE:
				case RAF_THAW:
					RAF_DEBUG("RAF_OBJ_GET | RAF_FREEZE/THAW");
					v = (void *)Freeze;
					break;

				case RAF_UNQ: {
					long max = 0;
					RAF_DEBUG("RAF_OBJ_GET | RAF_UNQ");
					for(i = 0; i < NumObj; i++)
						if(max <= ListCod[i]) 
							max = ListCod[i];
					max++;
					RAF_DEBUG("Numero unico: %04lu", max);
				  v = (void *)max;
					break;
				}
				break;
					
				case RAF_FIRST:
				case RAF_PREV :
				case RAF_NEXT :
				case RAF_LAST :
					RAF_DEBUG("RAF_OBJ_GET -> RAF_OBJ_SLT");
					/* RAF 2001-05-30: viene naturale usare GET invece di SLT 
					 * e poiche` non vale la pena di fare i  difficili redirigo 
					 * la chiamata GET alla SLT che ha i sub_flag chiamati.
					 */ 
					v = (void *)raf_obj_manage(RAF_OBJ_SLT | (flag & 0x0F), pAram);
					break;
				
				default:
					RAF_DEBUG("RAF_OBJ_GET default case");
					v = ListObj[Select];
					break;
			}
			break;
					
		default:
			v = ListObj[Select];
			break;
	}		
	
	return v;
}


